"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Node_class_1 = require("./Node.class");
const fs = require('fs');
const path = require('path');
const util = require('util');
const jsdom = require('jsdom');
class xjs_Document extends Node_class_1.default {
    static async fromFile(filepath) {
        let data = await util.promisify(fs.readFile)(filepath, 'utf8');
        return new xjs_Document(new jsdom.JSDOM(data).window.document);
    }
    static fromFileSync(filepath) {
        let data = fs.readFileSync(filepath, 'utf8');
        return new xjs_Document(new jsdom.JSDOM(data).window.document);
    }
    constructor(node) {
        super(node);
    }
    get node() { return super.node; }
    prepend(...contents) {
        this.node.prepend(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    append(...contents) {
        this.node.append(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    innerHTML() {
        let root = this.node.querySelector('html');
        if (root === null)
            throw new ReferenceError('No <html> element found.');
        return new jsdom.JSDOM('<!doctype html>' + root.outerHTML).serialize();
    }
    importLinks(dirpath) {
        const xjs_DocumentFragment = require('./DocumentFragment.class').default;
        const xjs_HTMLTemplateElement = require('./HTMLTemplateElement.class').default;
        if (!('import' in jsdom.JSDOM.fragment('<link rel="import" href="https://example.com/"/>').querySelector('link'))) {
            console.warn('`HTMLLinkElement#import` is not yet supported. Replacing `<link>`s with their imported contents.');
            this.node.querySelectorAll('link[rel~="import"][data-import]').forEach((link) => {
                const switch_ = {
                    'document': () => xjs_DocumentFragment.fromFileSync(path.join(dirpath, link.href)).node,
                    'template': () => xjs_HTMLTemplateElement.fromFileSync(path.join(dirpath, link.href)).content(),
                    default() { return null; },
                };
                let imported = (switch_[link.getAttribute('data-import')] || switch_.default).call(this);
                if (imported) {
                    link.after(imported);
                    link.remove();
                }
            });
        }
        return this;
    }
    async importLinksAsync(dirpath) {
        const xjs_DocumentFragment = require('./DocumentFragment.class').default;
        const xjs_HTMLTemplateElement = require('./HTMLTemplateElement.class').default;
        if (!('import' in jsdom.JSDOM.fragment('<link rel="import" href="https://example.com/"/>').querySelector('link'))) {
            console.warn('`HTMLLinkElement#import` is not yet supported. Replacing `<link>`s with their imported contents.');
            return Promise.all([...this.node.querySelectorAll('link[rel~="import"][data-import]')].map(async (link) => {
                const switch_ = {
                    'document': async () => (await xjs_DocumentFragment.fromFile(path.join(dirpath, link.href))).node,
                    'template': async () => (await xjs_HTMLTemplateElement.fromFile(path.join(dirpath, link.href))).content(),
                    async default() { return null; },
                };
                let imported = await (switch_[link.getAttribute('data-import')] || switch_.default).call(this);
                if (imported) {
                    link.after(imported);
                    link.remove();
                }
            }));
        }
        else
            return Promise.all([]);
    }
}
exports.default = xjs_Document;
