"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Node_class_1 = require("./Node.class");
const xjs = {
    Object: require('extrajs').Object,
};
class xjs_Element extends Node_class_1.default {
    constructor(node) {
        super(node);
    }
    get node() { return super.node; }
    get tagName() { return this.node.tagName.toLowerCase(); }
    innerHTML(markup) {
        if (arguments.length) {
            this.node.innerHTML = markup;
            return this;
        }
        return this.node.innerHTML;
    }
    outerHTML(markup) {
        if (arguments.length) {
            throw new Error('feature not supported yet');
        }
        return this.node.outerHTML;
    }
    prepend(...contents) {
        this.node.prepend(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    append(...contents) {
        this.node.append(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    attr(attr = '', value, this_arg = this) {
        switch (xjs.Object.typeOf(attr)) {
            case 'null': break;
            case 'string':
                if (attr.trim() === '')
                    throw new RangeError('Attribute name cannot be empty string.');
                switch (xjs.Object.typeOf(value)) {
                    case 'function': return this.attr(attr, value.call(this_arg));
                    case 'null':
                        this.node.removeAttribute(attr);
                        break;
                    case 'undefined': return this.node.getAttribute(attr);
                    default:
                        this.node.setAttribute(attr, value.toString());
                        break;
                }
                break;
            case 'object':
                for (let i in attr)
                    this.attr(i, attr[i]);
                break;
            default: break;
        }
        return this;
    }
    id(value, this_arg = this) {
        if (arguments.length) {
            if (xjs.Object.typeOf(value) === 'string')
                this.node.id = value;
            else
                this.attr('id', value, this_arg);
            return this;
        }
        else
            return this.node.id;
    }
    class(value, this_arg = this) {
        if (arguments.length) {
            if (xjs.Object.typeOf(value) === 'string')
                this.node.className = value;
            else
                this.attr('class', value, this_arg);
            return this;
        }
        else
            return this.node.className;
    }
    addClass(...tokens) {
        tokens.forEach((token) => {
            token.split(' ').forEach((t) => {
                if (t.trim() !== '')
                    this.node.classList.add(t);
            });
        });
        return this;
    }
    removeClass(...tokens) {
        tokens.forEach((token) => {
            token.split(' ').forEach((t) => {
                if (t.trim() !== '')
                    this.node.classList.remove(t);
            });
        });
        return this;
    }
    replaceClassString(old_, new_) {
        this.node.className = this.node.className.replace(old_, new_);
        return this;
    }
}
exports.default = xjs_Element;
