"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Element_class_1 = require("./Element.class");
const xjs = {
    Object: require('extrajs').Object,
};
class xjs_HTMLElement extends Element_class_1.default {
    constructor(node) {
        super(node);
    }
    get node() { return super.node; }
    title(val, this_arg = this) {
        return this.attr('title', val, this_arg);
    }
    lang(val, this_arg = this) {
        return this.attr('lang', val, this_arg);
    }
    dir(val, this_arg = this) {
        return this.attr('dir', val, this_arg);
    }
    hidden(val, this_arg = this) {
        return this.attr('hidden', val, this_arg);
    }
    tabIndex(val, this_arg = this) {
        return this.attr('tabindex', val, this_arg);
    }
    style(prop = '', value, this_arg = this) {
        switch (xjs.Object.typeOf(prop)) {
            case 'null': break;
            case 'string':
                if (prop.trim() === '')
                    throw new RangeError('Property name cannot be empty string.');
                switch (xjs.Object.typeOf(value)) {
                    case 'function': return this.style(prop, value.call(this_arg));
                    case 'undefined': return this.node.style.getPropertyValue(prop) || null;
                    default:
                        switch (value) {
                            case '':
                            case null:
                                this.node.style.removeProperty(prop);
                                break;
                            default:
                                this.node.style.setProperty(prop, value.toString());
                                break;
                        }
                }
                break;
            case 'object':
                for (let i in prop)
                    this.style(i, prop[i]);
                break;
            default: break;
        }
        return this;
    }
    data(data_attr = '', value, this_arg = this) {
        switch (xjs.Object.typeOf(data_attr)) {
            case 'null': break;
            case 'string':
                if (data_attr.trim() === '')
                    throw new RangeError('Data-Attribute name cannot be empty string.');
                switch (xjs.Object.typeOf(value)) {
                    case 'function': return this.data(data_attr, value.call(this_arg));
                    case 'null':
                        delete this.node.dataset[data_attr];
                        break;
                    case 'undefined':
                        let returned = this.node.dataset[data_attr];
                        return (xjs.Object.typeOf(returned) === 'string') ? returned : null;
                    default:
                        this.node.dataset[data_attr] = value.toString();
                        break;
                }
                break;
            case 'object':
                for (let i in data_attr)
                    this.data(i, data_attr[i]);
                break;
            default: break;
        }
        return this;
    }
}
exports.default = xjs_HTMLElement;
